@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: hostip.c,v 1.36 2001/12/04 13:03:09 bagder Exp $
+ * $Id: hostip.c,v 1.48 2002/01/18 12:59:33 bagder Exp $
  *****************************************************************************/
 
 #include "setup.h"
@@ -58,6 +58,10 @@
 #include "urldata.h"
 #include "sendf.h"
 #include "hostip.h"
+#include "hash.h"
+
+#define _MPRINTF_REPLACE /* use our functions only */
+#include <curl/mprintf.h>
 
 #if defined(HAVE_INET_NTOA_R) && !defined(HAVE_INET_NTOA_R_DECL)
 #include "inet_ntoa_r.h"
@@ -68,6 +72,151 @@
 #include "memdebug.h"
 #endif
 
+static curl_hash hostname_cache;
+static int host_cache_initialized;
+
+void Curl_global_host_cache_init(void)
+{
+  if (!host_cache_initialized) {
+    curl_hash_init(&hostname_cache, 7, Curl_freeaddrinfo);
+    host_cache_initialized = 1;
+  }
+}
+
+curl_hash *Curl_global_host_cache_get(void)
+{
+  return &hostname_cache;
+}
+
+void Curl_global_host_cache_dtor(void)
+{
+  if (host_cache_initialized) {
+    curl_hash_clean(&hostname_cache);
+    host_cache_initialized = 0;
+  }
+}
+
+struct curl_dns_cache_entry {
+  Curl_addrinfo *addr;
+  time_t timestamp;
+};
+
+/* count the number of characters that an integer takes up */
+static int _num_chars(int i)
+{
+  int chars = 0;
+
+  /* While the number divided by 10 is greater than one, 
+   * re-divide the number by 10, and increment the number of 
+   * characters by 1.
+   *
+   * this relies on the fact that for every multiple of 10, 
+   * a new digit is added onto every number
+   */
+  do {
+    chars++;
+
+    i = (int) i / 10;
+  } while (i > 1);
+
+  return chars;
+}
+
+/* Create a hostcache id */
+static char *
+_create_hostcache_id(char *server, int port, ssize_t *entry_len)
+{
+  char *id = NULL;
+
+  /* Get the length of the new entry id */
+  *entry_len = *entry_len +      /* Hostname length */
+               1 +               /* The ':' seperator */
+               _num_chars(port); /* The number of characters the port will take up */
+  
+  /* Allocate the new entry id */
+  id = malloc(*entry_len + 1);
+  if (!id) {
+    return NULL;
+  }
+
+  /* Create the new entry */
+  /* If sprintf() doesn't return the entry length, that signals failure */
+  if (sprintf(id, "%s:%d", server, port) != *entry_len) {
+    /* Free the allocated id, set length to zero and return NULL */
+    *entry_len = 0;
+    free(id);
+    return NULL;
+  }
+
+  return id;
+}
+
+/* Macro to save redundant free'ing of entry_id */
+#define _hostcache_return(__v) \
+{ \
+  free(entry_id); \
+  return (__v); \
+}
+
+Curl_addrinfo *Curl_resolv(struct SessionHandle *data,
+                           char *hostname,
+                           int port,
+                           char **bufp)
+{
+  char *entry_id = NULL;
+  struct curl_dns_cache_entry *p = NULL;
+  ssize_t entry_len;
+  time_t now;
+
+  /* If the host cache timeout is 0, we don't do DNS cach'ing
+     so fall through */
+  if (data->set.dns_cache_timeout == 0) {
+    return Curl_getaddrinfo(data, hostname, port, bufp);
+  }
+
+  /* Create an entry id, based upon the hostname and port */
+  entry_len = strlen(hostname);
+  entry_id = _create_hostcache_id(hostname, port, &entry_len);
+  /* If we can't create the entry id, don't cache, just fall-through
+     to the plain Curl_getaddrinfo() */
+  if (!entry_id) {
+    return Curl_getaddrinfo(data, hostname, port, bufp);
+  }
+  
+  time(&now);
+  /* See if its already in our dns cache */
+  if (entry_id && curl_hash_find(data->hostcache, entry_id, entry_len+1, (void **) &p)) {
+    /* Do we need to check for a cache timeout? */
+    if (data->set.dns_cache_timeout != -1) {
+      /* Return if the entry has not timed out */
+      if ((now - p->timestamp) < data->set.dns_cache_timeout) {
+        _hostcache_return(p->addr);
+      }
+    }
+    else {
+      _hostcache_return(p->addr);
+    }
+  }
+
+  /* Create a new cache entry */
+  p = (struct curl_dns_cache_entry *) malloc(sizeof(struct curl_dns_cache_entry));
+  if (!p) {
+   _hostcache_return(NULL);
+  }
+
+  p->addr = Curl_getaddrinfo(data, hostname, port, bufp);
+  if (!p->addr) {
+    free(p);
+    _hostcache_return(NULL);
+  }
+  p->timestamp = now;
+
+  /* Save it in our host cache */
+  curl_hash_update(data->hostcache, entry_id, entry_len+1, (const void *) p);
+
+  _hostcache_return(p->addr);
+}
+
 /*
  * This is a wrapper function for freeing name information in a protocol
  * independent way. This takes care of using the appropriate underlaying
@@ -75,11 +224,15 @@
  */
 void Curl_freeaddrinfo(void *freethis)
 {
+  struct curl_dns_cache_entry *p = (struct curl_dns_cache_entry *) freethis;
+
 #ifdef ENABLE_IPV6
-  freeaddrinfo(freethis);
+  freeaddrinfo(p->addr);
 #else
-  free(freethis);
+  free(p->addr);
 #endif
+
+  free(p);
 }
 
 /* --- resolve name or IP-number --- */
@@ -102,7 +255,7 @@ int curl_getaddrinfo(char *hostname, char *service,
     /* success */
     if(logfile)
       fprintf(logfile, "ADDR %s:%d getaddrinfo() = %p\n",
-              source, line, *result);
+              source, line, (void *)*result);
   }
   else {
     if(logfile)
@@ -118,7 +271,7 @@ void curl_freeaddrinfo(struct addrinfo *freethis,
   (freeaddrinfo)(freethis);
   if(logfile)
     fprintf(logfile, "ADDR %s:%d freeaddrinfo(%p)\n",
-            source, line, freethis);
+            source, line, (void *)freethis);
 }
 
 #endif
@@ -265,7 +418,7 @@ Curl_addrinfo *Curl_getaddrinfo(struct SessionHandle *data,
                                 char **bufp)
 {
   struct hostent *h = NULL;
-  unsigned long in;
+  in_addr_t in;
   int ret; /* this variable is unused on several platforms but used on some */
 
 #define CURL_NAMELOOKUP_SIZE 9000
@@ -273,10 +426,10 @@ Curl_addrinfo *Curl_getaddrinfo(struct SessionHandle *data,
    * everything. OSF1 is known to require at least 8872 bytes. The buffer
    * required for storing all possible aliases and IP numbers is according to
    * Stevens' Unix Network Programming 2nd editor, p. 304: 8192 bytes! */
-  char *buf = (char *)malloc(CURL_NAMELOOKUP_SIZE);
+  int *buf = (int *)malloc(CURL_NAMELOOKUP_SIZE);
   if(!buf)
     return NULL; /* major failure */
-  *bufp = buf;
+  *bufp = (char *)buf;
 
   port=0; /* unused in IPv4 code */
   ret = 0; /* to prevent the compiler warning */
@@ -306,7 +459,7 @@ Curl_addrinfo *Curl_getaddrinfo(struct SessionHandle *data,
     /* Solaris, IRIX and more */
     if ((h = gethostbyname_r(hostname,
                              (struct hostent *)buf,
-                             buf + sizeof(struct hostent),
+                             (char *)buf + sizeof(struct hostent),
                              CURL_NAMELOOKUP_SIZE - sizeof(struct hostent),
                              &h_errnop)) == NULL )
 #endif
@@ -372,3 +525,4 @@ Curl_addrinfo *Curl_getaddrinfo(struct SessionHandle *data,
  * vim600: fdm=marker
  * vim: et sw=2 ts=2 sts=2 tw=78
  */
+ 
