@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: http.c,v 1.80 2001/11/16 11:21:50 bagder Exp $
+ * $Id: http.c,v 1.85 2002/01/16 14:47:00 bagder Exp $
  *****************************************************************************/
 
 #include "setup.h"
@@ -128,8 +128,10 @@ static
 CURLcode add_buffer_send(int sockfd, struct connectdata *conn, send_buffer *in,
                          long *bytes_written)
 {
-  size_t amount;
-  CURLcode result;
+  ssize_t amount;
+  CURLcode res;
+  char *ptr;
+  int size;
 
   if(conn->data->set.verbose) {
     fputs("> ", conn->data->set.err);
@@ -137,7 +139,25 @@ CURLcode add_buffer_send(int sockfd, struct connectdata *conn, send_buffer *in,
     fwrite(in->buffer, in->size_used, 1, conn->data->set.err);
   }
 
-  result = Curl_write(conn, sockfd, in->buffer, in->size_used, &amount);
+  /* The looping below is required since we use non-blocking sockets, but due
+     to the circumstances we will just loop and try again and again etc */
+
+  ptr = in->buffer;
+  size = in->size_used;
+  do {
+    res = Curl_write(conn, sockfd, ptr, size, &amount);
+
+    if(CURLE_OK != res)
+      break;
+
+    if(amount != size) {
+      size += amount;
+      ptr += amount;
+    }
+    else
+      break;
+
+  } while(1);
 
   if(in->buffer)
     free(in->buffer);
@@ -145,7 +165,7 @@ CURLcode add_buffer_send(int sockfd, struct connectdata *conn, send_buffer *in,
 
   *bytes_written = amount;
 
-  return result;
+  return res;
 }
 
 
@@ -235,6 +255,7 @@ CURLcode Curl_ConnectHTTPProxyTunnel(struct connectdata *conn,
   int subversion=0;
   struct SessionHandle *data=conn->data;
   CURLcode result;
+  int res;
 
   int nread;   /* total size read */
   int perline; /* count bytes per line */
@@ -317,8 +338,12 @@ CURLcode Curl_ConnectHTTPProxyTunnel(struct connectdata *conn,
        * to read, but when we use Curl_read() it may do so. Do confirm
        * that this is still ok and then remove this comment!
        */
-      if(CURLE_OK != Curl_read(conn, tunnelsocket, ptr, BUFSIZE-nread,
-                               &gotbytes))
+      res= Curl_read(conn, tunnelsocket, ptr, BUFSIZE-nread,
+                     &gotbytes);
+      if(res< 0)
+        /* EWOULDBLOCK */
+        continue; /* go loop yourself */
+      else if(res)
         keepon = FALSE;
       else if(gotbytes <= 0) {
         keepon = FALSE;
@@ -449,7 +474,7 @@ CURLcode Curl_http_done(struct connectdata *conn)
   if(0 == (http->readbytecount + conn->headerbytecount)) {
     /* nothing was read from the HTTP server, this can't be right
        so we return an error here */
-    failf(data, "Empty reply from server\n");
+    failf(data, "Empty reply from server");
     return CURLE_GOT_NOTHING;
   }
 
@@ -610,7 +635,7 @@ CURLcode Curl_http(struct connectdata *conn)
 
         passed += actuallyread;
         if(actuallyread != readthisamountnow) {
-          failf(data, "Could only read %d bytes from the input\n",
+          failf(data, "Could only read %d bytes from the input",
                 passed);
           return CURLE_READ_ERROR;
         }
@@ -621,7 +646,7 @@ CURLcode Curl_http(struct connectdata *conn)
         data->set.infilesize -= conn->resume_from;
 
         if(data->set.infilesize <= 0) {
-          failf(data, "File already completely uploaded\n");
+          failf(data, "File already completely uploaded");
           return CURLE_PARTIAL_FILE;
         }
       }
@@ -735,10 +760,8 @@ CURLcode Curl_http(struct connectdata *conn)
        * equal to UTC (Coordinated Universal Time)." (see page 20 of RFC2616).
        */
 
-#ifdef HAVE_LOCALTIME_R
+#ifdef HAVE_GMTIME_R
       /* thread-safe version */
-      /* We assume that the presense of localtime_r() proves the presense
-         of gmtime_r() which is a bit ugly but might work */
       struct tm keeptime;
       thistime = (struct tm *)gmtime_r(&data->set.timevalue, &keeptime);
 #else
@@ -795,7 +818,7 @@ CURLcode Curl_http(struct connectdata *conn)
       char contentType[256];
       int linelength=0;
       if(Curl_FormInit(&http->form, http->sendit)) {
-        failf(data, "Internal HTTP POST error!\n");
+        failf(data, "Internal HTTP POST error!");
         return CURLE_HTTP_POST_ERROR;
       }
 
@@ -826,7 +849,7 @@ CURLcode Curl_http(struct connectdata *conn)
                                            1,
                                            (FILE *)&http->form);
         if(linelength == -1) {
-          failf(data, "Could not get Content-Type header line!\n");
+          failf(data, "Could not get Content-Type header line!");
           return CURLE_HTTP_POST_ERROR;
         }
         add_buffer(req_buffer, contentType, linelength);
